% This program calculates the ideal filter for a square passband
% and a linear taper in a specified transition band.  Set fp=fs
% to achieve conventional ideal square filter.
%
clear all
close all

fp = input('Please enter passband, max(|f|<=1.0) fp: ');
fs = input('Please enter fs: ');

if fp > fs
    error('Wrong fs or fp! fp should be less than fs')
end

%compute the ideal frequency response

[f1, f2] = freqspace(21, 'meshgrid');
Hd = zeros(21);
if fs > fp
    for i = 1:21
        for j = 1:21
            x(i, j) = (i - 11)/10;
            y(i, j) = (j - 11)/10;
            x1 = 0;
            x2 = 0;
            if abs(f1(1, j)) > fp & abs(f1(1, j)) < fs
                x1 = (fs - abs(f1(1,j)))/(fs-fp);
            elseif abs(f1(1, j)) <= fp
                x1 = 1;
            end
            
            if abs(f2(i, 1)) > fp & abs(f2(i, 1)) < fs
                x2 = (fs - abs(f2(i, 1)))/(fs-fp);
            elseif abs(f2(i, 1)) <= fp
                x2 = 1;
            end
            
            Hd(i, j) = x1*x2;
        end
    end
end
Hd((abs(f1) < fp) & (abs(f2) < fp)) = 1;

%plot the ideal frequency response

figure(1)
mesh([-10:10]/10, [-10:10]/10, Hd);
title('the ideal frequency response')
xlabel('f_1')
ylabel('f_2')

%compute the ideal impulse response

wp = fp * pi;
ws = fs * pi;
if wp == ws
   for i = -20 : 20
   for j = -20 : 20
      n1(21+i, 21+j) = i;
      n2(21+i, 21+j) = j;
      %Computing impluse response
      if i == 0
         h1 = wp/pi;
      else
      	h1 = sin(wp*i)/(i*pi);
      end
      if j == 0
         h2 = wp/pi;
      else
	      h2 = sin(wp*j)/(j*pi);
      end
      hd(21+i, 21+j) = h1*h2;
   end
	end
else %wp != ws
   for i = -20 : 20
   for j = -20 : 20
      n1(21+i, 21+j) = i;
      n2(21+i, 21+j) = j;
      
      %Computing impluse response
      if i == 0
         h1 = (ws+wp)/2;
      else
      	h1 = 2*sin((ws+wp)*i/2)*sin((ws-wp)*i/2)/((ws-wp)*i*i*pi);
      end
      if j == 0
         h2 = (ws+wp)/2;
      else
	      h2 = 2*sin((ws+wp)*j/2)*sin((ws-wp)*j/2)/((ws-wp)*j*j*pi);
      end
      hd(21+i, 21+j) = h1*h2;
   end
	end
end

%plot the ideal impulse response

figure(2) 
mesh(hd);
title('the ideal impulse response')
xlabel('n_1 +20')
ylabel('n_2 +20')

N = input('Please enter the size of Hamming window N: ');

%filter design with window method

h = fwind1(Hd, hamming(N));

figure(3)
freqz2(h);
title('the designed  frequency response')
xlabel('f_1')
ylabel('f_2')

figure(4)
mesh(h);
title('the designed impulse response')
xlabel('n_1 +shift')
ylabel('n_2 +shift')


